(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Pierre Weis, projet Cristal, INRIA Rocquencourt          *)
(*                                                                     *)
(*  Copyright 2000 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  Distributed only by permission.                   *)
(*                                                                     *)
(***********************************************************************)

(* Extra graphics functions primitives. *)
let nround x y = (2 * x + y) / (2 * y);;
let mxround x = nround (0xFF * x) 0xFF;;

let hsb_of_rgb r g b =
 let bri = max r (max g b)
 and min = min r (min g b) in
 let d = bri - min in
 let sat = if bri = 0 then 0 else nround (0xFF * d) bri in
 let hue =
  if bri = 0 || d = 0 then 0 else
  let r' = (bri - r) * 0xFF / d
  and g' = (bri - g) * 0xFF / d
  and b' = (bri - b) * 0xFF / d in
  let h =
   if bri = r then if min = g then 5 * 0xFF + b' else 0xFF - g' else
   if bri = g then if min = b then 0xFF + r' else 3 * 0xFF - b' else
   if min = r then 3 * 0xFF + g' else 5 * 0xFF - r' in
  h / 6 in
 hue, sat, bri;;

let rgb_of_hsb h s b =
 let h = h * 6 in
 let i = h / 0xFF * 0xFF in
 let f = h - i in
 let m = b * (0xFF - s) / 0xFF and n = b * (0xFF - s * f / 0xFF) / 0xFF
 and k = b * (0xFF - s * (0xFF - f) / 0xFF) / 0xFF in
 match i / 0xFF with
 | 0 | 6 -> mxround b, mxround k, mxround m
 | 1 -> mxround n, mxround b, mxround m
 | 2 -> mxround m, mxround b, mxround k
 | 3 -> mxround m, mxround n, mxround b
 | 4 -> mxround k, mxround m, mxround b
 | 5 -> mxround b, mxround m, mxround n
 | _ -> invalid_arg "rgb_of_hsb";;

let color_of_rgb r g b = (r lsl 16) + (g lsl 8) + b;;

let r_of_color c = (c lsr 16) land 0xFF;;
let g_of_color c = (c lsr 8) land 0xFF;;
let b_of_color c = c land 0xFF;;

let rgb_of_color c = (c lsr 16) land 0xFF, (c lsr 8) land 0xFF, c land 0xFF;;

let hsb_of_color c =
 hsb_of_rgb ((c lsr 16) land 0xFF) ((c lsr 8) land 0xFF) (c land 0xFF);;

open Graphics;;

type pixels = color array array;;

let draw_pixels pixels x y =
 let img = Graphics.make_image pixels in
 Graphics.draw_image img x y;;

let pixel_is_set ts tb c =
 let h, s, b = hsb_of_color c in
 b <= tb || s <= ts;;

let bitmap_of_pixels ts tb px =
 let l = Array.length px in
 if l = 0 then invalid_arg "make_transp" else
 let c = Array.length px.(0) in
 let res = Array.make_matrix l c 0 in
 for i = 0 to l - 1 do
  let line = px.(i) in
  let res_line = res.(i) in
  for j = 0 to c - 1 do
   let res_color =
    let color = line.(j) in
    if pixel_is_set ts tb color then white else black in
   res_line.(j) <- res_color    
  done
 done;
 res;;

(* Set pixels of an image which have background color to transparent color. *)
let set_background_transp px =
 let l = Array.length px in
 if l = 0 then invalid_arg "set_background_transp" else
 let c = Array.length px.(0) in
 for i = 0 to l - 1 do
  let line = px.(i) in
  for j = 0 to c - 1 do
   if line.(j) = background then line.(j) <- transp
  done
 done;;

